"use strict"

require("simple-errors");
var winston = require("winston");

var kidoConnector = require("kido-connector");
var kidoConnectorRest = require("kido-connector-rest");

var jiraRESTOps = require("./jira-api-methods");

// Using kido connector to configure winston
kidoConnector.init("node-jira-api", winston);

/*
* @param 'settings' {object} Required. Constructor options. It can have the following properties:
*   - 'restEndpoint' {object} Required. The endpoint rest that should be call at each method.
*   - 'headers' {object} Optional. Used as global headers to be added to all requests before execution.
*   - 'authType' {object} Optional. The authentication settings required by the authentication manager.
*/
function JiraConnector(config) {

	function createSession(credential, cb) {
		cb(null, { auth: credential.auth || (new Buffer(credential.username + ':' + credential.password).toString("base64")) });
	};

	var connector = new kidoConnector.Connector({
		config: config,
		credentialProps: ["username", "password", "auth"],
		createSessionCb: createSession,
		disableCache: true
	});

	var settings = {
		restEndpoint: config.restEndpoint, 
		headers 	: config.headers,
		restOps 	: jiraRESTOps,
		authContext : { 
			kidoConnector: connector, 
			authType: config.authType || "basic"
		},
		restHelper: config.customRestHelper
	};

	new kidoConnectorRest.RestClient(settings).hookMethods(this, function (options, cb) {
		// could update 'options' object is desired before callback
		cb(null, function (err, result, interceptCb) {
			// customize internal/server errors (to be "aligned" with JIRA's response error)
			if (err) {
				var customError = {
					errorMessages: [ err.toString() ],
					errors: err,
					statusCode: 500
				};
			}

			// customize result
			if (result) {
				// move errors to err object
				if (result.error) {
					var customError = typeof result.error === 'object' ? result.error : {
						errorMessages: [ result.error.toString() ],
						errors: { }
					};
					customError.statusCode = result.statusCode;
				}
				// if no error delete headers from result
				else {
					delete result.headers;
				}
			}

			interceptCb(customError, result);
		});
	});

	this.authenticate = function (options, cb) {
		connector.getSession(options, function (err, session, auth) {
			cb(err, { auth: auth });
		});
	};
};

module.exports = JiraConnector;