var assert = require("assert");

var JiraConnector = require("../index");

describe('JiraConnector', function () {
    describe("constructor", function () {
        it("should hook all API methods to instance", function (done) {
            // arrange
            var apiOpsCount = 203;

            var targetFunctions = 0, 
                targetProperties = 0, 
                targetOtherProperties = 0;

            // act
            var settings = {};
        	var jc = new JiraConnector(settings);
        	
            // assert
        	assert.ok(jc);
        	
        	for (var prop in jc) {
        		if (jc.hasOwnProperty(prop)) {
        			if (typeof jc[prop] === 'function') {
        				targetFunctions++;
        			} else {
        				targetProperties++;
        			}
        		} else {
        			targetOtherProperties++;
        		}
        	}

        	assert.equal(targetFunctions, apiOpsCount + 1);
        	assert.equal(targetProperties, 0);
        	assert.equal(targetOtherProperties, 0);

        	done();
        });

        it("should load & use default JIRA operations", function (done) {
            // arrange
            var settings = {};

            var jiraRESTOps = require("../jira-api-methods");
            var groups = Object.keys(jiraRESTOps);

            // act
            var jc = new JiraConnector(settings);
            
            // assert
            assert.ok(jc);

            for (var i = 0; i < groups.length; i++) {
                var methods = Object.keys(jiraRESTOps[groups[i]]);
                for (var j = 0; j < methods.length; j++) {
                    assert.equal(typeof jc[methods[j]], 'function');
                }
            }

            done();
        });        
    });

    describe("authentication", function() {
        it("should return basic authentication token", function (done) {
            // arrange
            var settings = {};

            var options = {
                username: 'john doe',
                password: 'secret'
            };

            var targetToken = new Buffer(options.username + ':' + options.password).toString("base64");

            var settings = {};
            var jc = new JiraConnector(settings);
            
            // assert
            assert.ok(jc);            

            // act
            jc.authenticate(options, function (err, result) {
                // assert
                assert.ok(!err);

                assert.equal(result.auth, targetToken);

                done();
            });
        });
    });

    describe("wrapping & intercepting results", function() {
        it("should convert error to JIRA's error simil", function (done) {
            // arrange
            var targetFunctionHit = false;

            var targetFunction = function (options, cb) {
                targetFunctionHit = true;
                cb("an error", null);
            };

            var customRestHelper = {
                getApiCallFcn: function (methodName, info) {
                    return targetFunction;
                }
            };

            var settings = {
                customRestHelper: customRestHelper
            };

            var jc = new JiraConnector(settings);

            // act (call any valid method in the dictionary)
            jc.issue(null, function (err, result) {
                // assert
                assert.ok(targetFunctionHit); 

                assert.equal(err.errorMessages[0], "an error");
                assert.equal(err.errors, "an error");
                assert.equal(err.statusCode, 500);

                assert.ok(!result);

                done();            
            });
        });

        it("should move object error at result to JIRA's error simil", function (done) {
            // arrange
            var targetFunctionHit = false;
            var targetResult = { 
                statusCode: 500,
                error: { 
                    ECode: 4931,
                    EMessage: "an error with error code"
                } 
            };

            var targetFunction = function (options, cb) {
                targetFunctionHit = true;
                cb(null, targetResult);
            };

            var customRestHelper = {
                getApiCallFcn: function (methodName, info) {
                    return targetFunction;
                }
            };

            var settings = {
                customRestHelper: customRestHelper
            };

            var jc = new JiraConnector(settings);

            // act (call any valid method in the dictionary)
            jc.issue(null, function (err, result) {
                // assert
                assert.ok(targetFunctionHit); 

                assert.equal(err, targetResult.error);
                assert.equal(err.ECode, 4931);
                assert.equal(err.EMessage, "an error with error code");
                assert.equal(err.statusCode, 500);

                assert.equal(result, targetResult);
    
                done();            
            });
        });


        it("should move non-object error at result to JIRA's error simil", function (done) {
            // arrange
            var targetFunctionHit = false;
            var targetResult = { 
                statusCode: 500,
                error: "an error at result" 
            };

            var targetFunction = function (options, cb) {
                targetFunctionHit = true;
                cb(null, targetResult);
            };

            var customRestHelper = {
                getApiCallFcn: function (methodName, info) {
                    return targetFunction;
                }
            };

            var settings = {
                customRestHelper: customRestHelper
            };

            var jc = new JiraConnector(settings);

            // act (call any valid method in the dictionary)
            jc.issue(null, function (err, result) {
                // assert
                assert.ok(targetFunctionHit); 

                assert.equal(err.errorMessages[0], "an error at result");
                assert.equal(typeof err.errors, 'object');
                assert.equal(err.statusCode, 500);

                assert.equal(result, targetResult);

                done();            
            });
        });

        it("should remove headers at successful result", function (done) {
            // arrange
            var targetFunctionHit = false;
            var targetResult = { 
                statusCode: 200,
                headers: { } 
            };

            var targetFunction = function (options, cb) {
                targetFunctionHit = true;
                cb(null, targetResult);
            };

            var customRestHelper = {
                getApiCallFcn: function (methodName, info) {
                    return targetFunction;
                }
            };

            var settings = {
                customRestHelper: customRestHelper
            };

            var jc = new JiraConnector(settings);

            // act (call any valid method in the dictionary)
            jc.issue(null, function (err, result) {
                // assert
                assert.ok(targetFunctionHit); 

                assert.ok(!err);

                assert.equal(result, targetResult);
                assert.equal(result.statusCode, 200);
                assert.ok(!result.headers);

                done();            
            });
        });
    });
});